/// @author: iMujtaba Nazki
/// Date: 15-04-2019
/// Copyright: im8488 @: 2019

import 'package:flutter/material.dart';

/// An InfoListItem is a StatelessWidget which is typically used to display
/// information within a ListView. An InfoListItem consists of a main container
/// called 'holder' which holds every other component together within an
/// InfoListItem. In terms of components, it consists of an resizable icon on
/// the left, and title, subTitle, and Footer text in top-down order on the
/// right. Depending on the size of the icon on the left other components
/// automatically resize themselves to fit the screen. Both the [titleText] and
/// [footerText] can display information upto 2 lines, after which elipsis are
/// displayed for the rest of the text, if any. The [subTitleText] can display
/// unlimited number of lines of text, depending on the user requirement. Color,
/// fontFamily whereever relevant of each component can be customized, except
/// for other properties such as fontSize, fontWeight, etc. which are built it,
/// and hence cannot be altered.
class InfoListItem extends StatelessWidget {
  final Color _holderColor;
  final Icon _icon;
  final String _titleText;
  final String _subTitleText;
  final String _footerText;
  final String _titleFontFamily;
  final String _subTitleFontFamily;
  final String _footerFontFamily;
  final Color _titleTextColor;
  final Color _subTitleTextColor;
  final Color _footerTextColor;

  // Used only internally.
  final double _leadingIconPadding = 5.0;

  /// Constructs an InfoListItem, and is the most customizable constructor than
  /// any other within the InfoListItem class. In fact, the other constructors
  /// are just redirecting constructors i.e. they redirect to this constructor
  /// for setting values of the properties. Most of the properites have default
  /// values such as an icon defaults to broken_image with 32px of size, the
  /// default background color is black, etc.
  InfoListItem({
    Icon icon = const Icon(Icons.broken_image, size: 32.0, color: Colors.amber),
    Color holderColor = Colors.black,
    String titleText = 'titleText',
    String subTitleText = 'subTitleText',
    String footerText = 'footerText',
    String titleFontFamily,
    String subTitleFontFamily,
    String footerFontFamily,
    Color titleTextColor = Colors.green,
    Color subTitleTextColor = Colors.white,
    Color footerTextColor = Colors.green,
  })  : _holderColor = holderColor,
        _icon = icon,
        _titleText = titleText,
        _subTitleText = subTitleText,
        _footerText = footerText,
        _titleFontFamily = titleFontFamily,
        _subTitleFontFamily = subTitleFontFamily,
        _footerFontFamily = footerFontFamily,
        _titleTextColor = titleTextColor,
        _subTitleTextColor = subTitleTextColor,
        _footerTextColor = footerTextColor;

  /// This method is called every time when there is a need to redraw the
  /// InfoListItem on the screen.
  @override
  Widget build(BuildContext context) {
    // It is quite possible, that this widget is wrapped within another widget,
    // say padding, in that case, to make the holder and its components fit the
    // screen automatically, it is wrapped within a FittedBox.
    return FittedBox(
      fit: BoxFit.contain,

      // SafeArea is used because of the thin bezels problem on the IOS device.
      child: SafeArea(
        child: _holder(context),
      ),
    );
  }

  /// [_holder] is the main container which holds everything else within an
  /// InfoListItem. Its width is the width of the deviceScreen, and has a
  /// padding of 5px on each side. It has a horizontal borderRadius with 30px
  /// on right as circular.
  Widget _holder(BuildContext context) {
    return Container(
      width: MediaQuery.of(context).size.width,
      margin: const EdgeInsets.all(5.0),
      decoration: BoxDecoration(
        color: _holderColor,
        borderRadius: BorderRadius.horizontal(
          left: Radius.zero,
          right: Radius.circular(30.0),
        ),
      ),
      child: Row(
        children: <Widget>[
          _leadingIcon(),
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              _title(context),
              _subTitle(context),
              _footer(context),
            ],
          ),
        ],
      ),
    );
  }

  /// The icon which is displayed towards the left of the InfoListItem.
  /// The size of its container is the size of the icon plus a padding of
  /// [_leadingIconPadding] on each side of the icon.
  Widget _leadingIcon() {
    return Container(
      width: _icon.size,
      padding: EdgeInsets.all(_leadingIconPadding),
      child: _icon,
    );
  }

  /// A [_title] can display at the most 2 lines of text. The container which
  /// contains the [_titleText] has a padding of 2.0, 35.0, 2.0, 20.0 on left,
  /// top, right, and bottom, respectively. [_titleText] has a fixed fontSize of
  /// 18px with bold FontWeight, which are not customizable. Though, other
  /// properties such as fontFamily, color, etc are customizable. TextOverflow
  /// is represented using elipsis, which again is preconfigured, and hence, not
  /// customizable.
  Widget _title(BuildContext context) {
    return _customContainer(
      context: context,
      padding: EdgeInsets.only(
        top: 2.0,
        right: 35.0,
        bottom: 2.0,
        left: 20.0,
      ),
      child: Text(
        _titleText,
        style: TextStyle(
          fontSize: 18.0,
          fontWeight: FontWeight.bold,
          fontFamily: _titleFontFamily,
          color: _titleTextColor,
        ),
        maxLines: 2,
        softWrap: true,
        overflow: TextOverflow.ellipsis,
      ),
    );
  }

  /// A [_subTitle] can display unlimited lines of text. The container which
  /// contains the [_subTitleText] has a padding of 2.0, 20.0, 2.0, 20.0 on left,
  /// top, right, and bottom, respectively. [_subTitleText] has a fixed fontSize
  /// of 16px with regular FontWeight, which are not customizable. Though, other
  /// properties such as fontFamily, color, etc are customizable. TextOverflow
  /// is represented using elipsis, which again is preconfigured, and hence,
  /// not customizable.
  Widget _subTitle(BuildContext context) {
    return _customContainer(
      context: context,
      padding: EdgeInsets.only(
        top: 2.0,
        right: 20.0,
        bottom: 2.0,
        left: 20.0,
      ),
      child: Text(
        _subTitleText,
        softWrap: true,
        // textAlign: TextAlign.justify,
        style: TextStyle(
          color: _subTitleTextColor,
          fontFamily: _subTitleFontFamily,
          fontSize: 16.0,
        ),
      ),
    );
  }

  /// A [_footer] like [_title] can display at the most 2 lines of text. The
  /// container which contains the [_footerText] has a padding of 10.0, 20.0,
  /// 10.0, 20.0 on left, top, right, and bottom, respectively. [_titleText] has
  /// a fixed fontSize of 13px with regular FontWeight, which are not
  /// customizable. Though, other properties such as fontFamily, color, etc are
  /// customizable. TextOverflow is represented using elipsis, which again is
  /// preconfigured, and hence, not customizable.
  Widget _footer(BuildContext context) {
    return _customContainer(
      context: context,
      padding: EdgeInsets.only(
        top: 10.0,
        right: 20.0,
        bottom: 10.0,
        left: 20.0,
      ),
      child: Text(
        _footerText,
        maxLines: 2,
        softWrap: true,
        textAlign: TextAlign.justify,
        overflow: TextOverflow.ellipsis,
        style: TextStyle(
          fontFamily: _footerFontFamily,
          color: _footerTextColor,
          fontSize: 13.0,
        ),
      ),
    );
  }

  /// A [_customContainer] is used internally in place of a container. It is the
  /// container that each component wraps itself in before it is placed within
  /// the holder. It is used because, a [customWidth] is applied to it, which is
  /// calculated by getting the size of the deviceScreen, and then subtracting
  /// that with the size of the icon on the left plus its left and right padding
  /// i.e. [_leadingIconPadding * 2].
  Widget _customContainer(
      {@required BuildContext context,
      EdgeInsets padding,
      @required Widget child}) {
    // The customWidth calculated based on total device width minus the iconSize
    // and its padding on left & right.
    double customWidth = MediaQuery.of(context).size.width -
        (_icon.size + (_leadingIconPadding * 2));

    return Container(
      width: customWidth,
      padding: padding,
      child: child,
    );
  }

  /////////////////////////// PRE-CONFIGURED ///////////////////////////////////

  /// Creates an InfoListItem with dark background color, white titleText color,
  /// amber subTitleText color, and white footerTextColor
  InfoListItem.dark({
    Icon icon = const Icon(Icons.broken_image, size: 32.0, color: Colors.white),
    String titleText = 'When things go...',
    String subTitleText =
        'Wrong as they sometimes will, When the road you are trudging seems all uphill',
    String footerText = '15 December, 2019',
    String titleFontFamily,
    String subTitleFontFamily,
    String footerFontFamily,
  }) : this(
          icon: icon,
          titleText: titleText,
          subTitleText: subTitleText,
          footerText: footerText,
          titleFontFamily: titleFontFamily,
          subTitleFontFamily: subTitleFontFamily,
          footerFontFamily: footerFontFamily,

          // Pre-configured.
          holderColor: Colors.black,
          titleTextColor: Colors.white,
          subTitleTextColor: Colors.amber,
          footerTextColor: Colors.white,
        );

  /// Creates an InfoListItem with black background color, deepPurple titleText
  /// color, white subTitleText color, and deepPurple footerTextColor
  InfoListItem.dark3d({
    Icon icon = const Icon(Icons.broken_image, size: 32.0, color: Colors.white),
    String titleText = 'Never look back...',
    String subTitleText =
        'you said, how was I to know, I love you so, Loniless up ahead, emptiness behind, where do I go, and you did\'t hear, all my...',
    String footerText = '15 December, 2019',
    String titleFontFamily,
    String subTitleFontFamily,
    String footerFontFamily,
  }) : this(
          icon: icon,
          titleText: titleText,
          subTitleText: subTitleText,
          footerText: footerText,
          titleFontFamily: titleFontFamily,
          subTitleFontFamily: subTitleFontFamily,
          footerFontFamily: footerFontFamily,

          // Pre-configured.
          holderColor: Colors.black,
          titleTextColor: Colors.deepPurple,
          subTitleTextColor: Colors.white,
          footerTextColor: Colors.deepPurple,
        );

  /// Creates an InfoListItem with black background color, yellow titleText color,
  /// white subTitleText color, and yellow footerTextColor
  InfoListItem.darkPro({
    Icon icon = const Icon(Icons.broken_image, size: 32.0, color: Colors.white),
    String titleText = 'Never look back...',
    String subTitleText =
        'you said, how was I to know, I love you so, Loniless up ahead, emptiness behind, where do I go, and you did\'t hear, all my...',
    String footerText = '15 December, 2019',
    String titleFontFamily,
    String subTitleFontFamily,
    String footerFontFamily,
  }) : this(
          icon: icon,
          titleText: titleText,
          subTitleText: subTitleText,
          footerText: footerText,
          titleFontFamily: titleFontFamily,
          subTitleFontFamily: subTitleFontFamily,
          footerFontFamily: footerFontFamily,

          // Pre-configured.
          holderColor: Colors.black,
          titleTextColor: Colors.yellow,
          subTitleTextColor: Colors.white,
          footerTextColor: Colors.yellow,
        );

  /// Creates an InfoListItem with deepPurple background color, white titleText
  /// color, lime subTitleText color, and white footerTextColor
  InfoListItem.purplePro({
    Icon icon = const Icon(Icons.broken_image, size: 32.0, color: Colors.white),
    String titleText = 'Never look back...',
    String subTitleText =
        'you said, how was I to know, I love you so, Loniless up ahead, emptiness behind, where do I go, and you did\'t hear, all my...',
    String footerText = '15 December, 2019',
    String titleFontFamily,
    String subTitleFontFamily,
    String footerFontFamily,
  }) : this(
          icon: icon,
          titleText: titleText,
          subTitleText: subTitleText,
          footerText: footerText,
          titleFontFamily: titleFontFamily,
          subTitleFontFamily: subTitleFontFamily,
          footerFontFamily: footerFontFamily,

          // Pre-configured.
          holderColor: Colors.deepPurple,
          titleTextColor: Colors.white,
          subTitleTextColor: Colors.lime,
          footerTextColor: Colors.white,
        );

  /// Creates an InfoListItem with white70 background color, blue titleText color,
  /// black subTitleText color, and green footerTextColor.
  InfoListItem.whitePro({
    Icon icon = const Icon(Icons.broken_image, size: 32.0, color: Colors.green),
    String titleText = 'Never look back...',
    String subTitleText =
        'you said, how was I to know, I love you so, Loniless up ahead, emptiness behind, where do I go, and you did\'t hear, all my...',
    String footerText = '15 December, 2019',
    String titleFontFamily,
    String subTitleFontFamily,
    String footerFontFamily,
  }) : this(
          icon: icon,
          titleText: titleText,
          subTitleText: subTitleText,
          footerText: footerText,
          titleFontFamily: titleFontFamily,
          subTitleFontFamily: subTitleFontFamily,
          footerFontFamily: footerFontFamily,

          // Pre-configured.
          holderColor: Colors.white70,
          titleTextColor: Colors.blue,
          subTitleTextColor: Colors.black,
          footerTextColor: Colors.green,
        );
}
